﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Xml;
using Chen.Common;
using Chen.DB;
using Chen.Ext;
using Newtonsoft.Json;
using HtmlAgilityPack;
namespace CnBlogsCHM
{
    class Program
    {
        //系统自带图片
        static string sysImage = "ContractedBlock.gif, ExpandedBlockStart.gif,copycode.gif";
        const string category = "category";//分类
        const string archive = "archive";//随笔
        static string title = "我的CHM";//CHM标题
        static int imgIndex = 1;//图片数量(作为图片文件名使用)
        static int fileIndex = 1;//文件数量
        static Dictionary<string, string> dicConvert = new Dictionary<string, string>();
        //Main
        static void Main(string[] args)
        {
            Console.WriteLine("将博客备份的xml拖到此处或者直接输入博客ID，回车即可");
            var input = Console.ReadLine().TrimEnd();
            bool ret;
            //创建html存储根目录
            Directory.CreateDirectory("./cnblogs/image");
            //复制content文件夹到根目录下
            DirectoryUtil.CopyDir(".//content", ".//cnblogs//content");
            //根据输入
            if (File.Exists(input))
                ret = ReadCNBlogsXml(input);
            else
                ret = ReadBlogsByID(input);
            if (!ret)
            {
                Console.WriteLine("程序异常");
                return;
            }
            //编译CHM文档
            ChmHelp chm = new ChmHelp();
            chm.RootPath = ".//cnblogs";
            chm.ChmFileName = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Desktop), title + ".chm");
            chm.DefaultPage = "博文目录.html";
            chm.Title = title;
            chm.dicConvert = dicConvert;
            chm.Compile();
            Console.WriteLine("生成完毕 文件位于" + chm.ChmFileName);
            Directory.Delete(".//cnblogs", true);
            Console.WriteLine("下载文件数量:{0}个 下载图片数量:{1}个", fileIndex, imgIndex);
            Console.ReadKey();
        }

        #region 得到博客对象
        static bool ReadCNBlogsXml(string path)
        {
            try
            {
                var xml = File.ReadAllText(path);//该xml为博客园随笔备份文件
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(xml);
                //获取rss节点下的内容
                var channelXml = doc.SelectSingleNode("rss").InnerXml;
                //进一步细化xml格式，内容仅为rss节点下的内容
                doc.LoadXml(channelXml);
                //判断item对象个数
                var items = doc.SelectNodes("channel/item");
                if (items.Count == 1)//仅当随笔内容有1项时才会多此一举
                {
                    //1.如果仅有一个item,序列化后的json格式为 "item": {"title": "测试","link":"",....} 这样的json格式并不能序列化成List<item>的
                    //刻意去添加一个空节点 在序列化成json的格式为 "item":[{"title": "测试"...},{}...] 这样就可以序列化成List<item>了
                    var item = doc.CreateElement("item");
                    doc.FirstChild.AppendChild(item);
                }
                //将xml序列化成json，并且去掉根节点
                var json = JsonConvert.SerializeXmlNode(doc, Newtonsoft.Json.Formatting.Indented, true);
                var channel = JsonConvert.DeserializeObject<Channel>(json);
                title = channel.title;
                //读取的xml没有类别名称
                foreach (var item in channel.items)
                {
                    item.typeName = DateTime.Parse(item.pubDate).ToString("yyyy年MM月");
                    //去掉标题中的#
                    item.title = item.title.RemoveFileInvalid();
                }

                #region 准备内容的html文件
                //加载html模板
                var template = File.ReadAllText(".//temChild.html");
                //定义目录表 结构
                DataTable dt = new DataTable();
                dt.Columns.Add("序号", typeof(int));
                dt.Columns.Add("标题", typeof(string));
                dt.Columns.Add("创建时间", typeof(string));
                foreach (var item in channel.items)
                {
                    item.description.content = DownImage(item.description.content);
                    //得到单个html正文
                    //以创建时间为目录分类html 
                    var time = DateTime.Parse(item.pubDate);
                    var content = template
                        .Replace("{channelTitle}", item.title)//博文标题
                        .Replace("{preContent}", item.description.content)//博文内容
                        .Replace("{channelHref}", item.link)//博文地址
                        .Replace("{channelLink}", channel.link)//博客地址
                        .Replace("{channelAuthor}", item.author);//博文作者
                    var dir = ".//cnblogs//" + item.typeName;
                    Directory.CreateDirectory(dir);
                    //html的文件名
                    File.WriteAllText(Path.Combine(dir, item.title + ".html"), content, Encoding.UTF8);
                    //处理完毕后 将该文件添加至目录表
                    var dr = dt.NewRow();
                    dr["序号"] = dt.Rows.Count + 1;
                    dr["标题"] = "<a href=\"{0}/{1}.html\"'>{1}</a>".FormatString(item.typeName, item.title);
                    dr["创建时间"] = DateTime.Parse(item.pubDate).ToString("yyyy-MM-dd HH:mm:ss");
                    dt.Rows.Add(dr);
                }
                //创建目录页
                dt.TableName = "<h3>博文目录<h3>";
                DbCommon.CreateHtml(dt, false, ".//cnblogs//博文目录.html");
                #endregion

                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }
        #endregion

        #region 结构定义
        public class Channel
        {
            public string title { get; set; }
            public string link { get; set; }
            public string description { get; set; }
            public string language { get; set; }
            public string lastBuildDate { get; set; }
            public string pubDate { get; set; }
            public string ttl { get; set; }
            [JsonProperty("item")]
            public List<Channel_Item> items { get; set; }
        }

        public class Channel_Item
        {
            public string title { get; set; }
            public string link { get; set; }
            public string author { get; set; }
            public string pubDate { get; set; }
            public string guid { get; set; }
            public Item_Description description { get; set; }
            [JsonIgnore]
            public string typeName { get; set; }
        }

        public class Item_Description
        {
            //默认以变量名称作为json序列化的节点，由于该节点内容不符合变量定义规范，则显示指定即可
            [JsonProperty("#cdata-section")]
            public string content { get; set; }
        }
        #endregion

        static bool ReadBlogsByID(string userId)
        {
            string url = "http://www.cnblogs.com/" + userId + "/mvc/blog/sidecolumn.aspx";//获取博客分类地址
            Console.WriteLine("正在连接服务器，请稍候...");
            //模版设置
            string divFormat = "<div style='width:100%;float:left;margin-top:20px;background-color:#999999;font-size:20px;color:White;padding-left:5px'>{0}</div>";
            string liFormat = "<li style='width:49%;float:left;line-height:30px;'><a href='{0}' target='_self' title='{1}'>{1}</a></li>";
            string template = File.ReadAllText(".//temChild.html");
            StringBuilder strIndex = new StringBuilder();//目录页
            //得到显示所有分类的页面 匹配出所有链接
            MatchCollection matchsCategory = Filter(url);
            if (matchsCategory == null)
            {
                Console.WriteLine("未找到任何类别，请检查UserID是否正确");
                return false;
            }
            foreach (Match mCategory in matchsCategory)
            {
                //获取类别信息
                var categoryName = mCategory.Groups["text"].Value.RemoveDirInvalid();
                var categoryUrl = mCategory.Groups["url"].Value;
                var lstCategory = new List<string>();
                //categoryUrl = "http://www.cnblogs.com/hlxs/category/382271.html";
                if (categoryUrl.IndexOf(userId + "/" + category, StringComparison.OrdinalIgnoreCase) >= 0 && !lstCategory.Contains(categoryUrl))
                {
                    Console.WriteLine();
                    Console.WriteLine("正在读取类别: {0}", mCategory.Groups["text"]);
                    lstCategory.Add(categoryUrl);
                    //将该类别添加到目录
                    strIndex.AppendLine(divFormat.FormatString(mCategory.Groups["text"].Value));
                    strIndex.AppendLine("<ul style='padding-top:10px;clear:both;list-style:none;'>");
                    //以类别名称创建目录
                    Directory.CreateDirectory(".//cnblogs//" + categoryName);
                    var lstArchive = new List<string>();
                    //获取该类别下的所有随笔页面内容，得到每个随笔链接
                    MatchCollection matchsArchive = Filter(categoryUrl);
                    foreach (Match mArchive in matchsArchive)
                    {
                        //随笔链接
                        var articleUrl = mArchive.Groups["url"].ToString();
                        if (articleUrl.IndexOf(userId + "/" + archive, StringComparison.OrdinalIgnoreCase) >= 0 && articleUrl.IndexOf('#') < 0 && !lstArchive.Contains(articleUrl))
                        {
                            lstArchive.Add(articleUrl);
                            var text = mArchive.Groups["text"].ToString().RemoveFileInvalid();
                            var autoName = (fileIndex++).ToString();
                            //打印提示信息
                            Console.WriteLine("正在下载: {0}", mArchive.Groups["text"].ToString());
                            //下载随笔内容 替换后保存本地
                            var contentCode = GetContent(articleUrl);//获取随笔内容
                            HtmlDocument htmlCode = new HtmlDocument();
                            htmlCode.LoadHtml(contentCode);
                            var titleNode = htmlCode.GetElementbyId("cb_post_title_url");
                            var postBody = htmlCode.GetElementbyId("cnblogs_post_body");
                            var postDate = htmlCode.GetElementbyId("post-date");
                            //var topics = htmlCode.GetElementbyId("topics");
                            if (titleNode == null || postBody == null || postDate == null)
                            {
                                Console.WriteLine("     该随笔获取异常，随笔名称:{0} 随笔链接:{1} 请检查是否设置了只允许注册用户访问，或其他");
                                continue;
                            }
                            var localHtml = template
                       .Replace("{channelTitle}", titleNode.InnerText)//博文标题
                       .Replace("{preContent}", DownImage(postBody.InnerHtml))//博文内容
                       .Replace("{channelHref}", titleNode.GetAttributeValue("href", "#"))//博文地址
                       .Replace("{channelLink}", userId + ".cnblogs.com")//博客地址
                       .Replace("{channelAuthor}", userId);//博文作者
                            var fileName = "./cnblogs/" + categoryName + "/" + autoName + ".html";
                            File.WriteAllText(fileName, localHtml,Encoding.UTF8);
                            //添加目录项
                            var relativePath = categoryName + "/" + autoName + ".html";
                            strIndex.AppendLine(liFormat.FormatString(relativePath, text));
                            //添加该文件的友好名称
                            dicConvert.Add(Path.GetFullPath(fileName), mArchive.Groups["text"].ToString());
                        }
                    }
                    //随笔类别结束UL标记
                    strIndex.AppendLine("</ul>");
                }
            }
            //创建完毕后写入目录
            if (strIndex.Length == 0)
            {
                Console.WriteLine("抓取不到这个博客的随笔分类！");
                return false;
            }
            //更换模版
            template = File.ReadAllText(".//temChild.html");
            var content = template.Replace("<a href='../博文目录.html' style='float: right; margin-top: 6px;'>返回目录</a>", string.Empty)
    .Replace("{channelTitle}", userId + " 随笔目录")//博文标题
    .Replace("{preContent}", strIndex.ToString())//博文内容
    .Replace("{channelHref}", "#")//博文地址
    .Replace("{channelLink}", userId + ".cnblogs.com")//博客地址
    .Replace("{channelAuthor}", userId);//博文作者
            File.WriteAllText(".//cnblogs//博文目录.html", content, Encoding.UTF8);
            title = userId + " 随笔目录";
            return true;
        }

        #region 辅助方法
        // 提取网页文件中的图片链接
        public static string[] GetHtmlImageUrls(string sHtmlText)
        {
            // 定义正则表达式用来匹配 img 标签     
            Regex regImg = new Regex(@"<img\b[^<>]*?\bsrc[\s\t\r\n]*=[\s\t\r\n]*[""']?[\s\t\r\n]*(?<imgUrl>[^\s\t\r\n""'<>]*)[^<>]*?/?[\s\t\r\n]*>", RegexOptions.IgnoreCase);
            // 搜索匹配的字符串      
            MatchCollection matches = regImg.Matches(sHtmlText);
            int i = 0;
            string[] sUrlList = new string[matches.Count];
            // 取得匹配项列表        
            foreach (Match match in matches)
                sUrlList[i++] = match.Groups["imgUrl"].Value;
            return sUrlList;
        }
        //获取页面的内容
        private static string GetContent(string url, string param = "")
        {
            try
            {
                if (!url.StartsWith("http://", StringComparison.OrdinalIgnoreCase))
                {
                    url = "http://" + url;
                }
                HttpWebRequest req = (HttpWebRequest)HttpWebRequest.Create(url);
                if (!string.IsNullOrEmpty(param))
                {
                    req.Method = "POST";
                    req.ContentType = "application/x-www-form-urlencoded";
                    byte[] bs = Encoding.ASCII.GetBytes(param);
                    req.ContentLength = bs.Length;
                    using (Stream reqStream = req.GetRequestStream())
                    {
                        reqStream.Write(bs, 0, bs.Length);
                    }
                }

                HttpWebResponse HttpWResp = (HttpWebResponse)req.GetResponse();
                using (Stream myStream = HttpWResp.GetResponseStream())
                {
                    if (myStream == null)
                    {
                        return string.Empty;
                    }
                    using (StreamReader sr = new StreamReader(myStream, Encoding.UTF8))
                    {
                        if (sr.Peek() > 0)
                        {
                            return sr.ReadToEnd();
                        }
                    }
                    return string.Empty;
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return null;
            }
        }
        //过滤
        static MatchCollection Filter(string url)
        {
            string content = GetContent(url);
            if (string.IsNullOrEmpty(content))
            {
                return null;
            }
            string regex = @"<a(?:(?!href=).)*href=(['""]?)(?<url>[^""\s>]*)\1[^>]*>(?<text>(?:(?!</?a\b).)*)</a>";
            Regex rx = new Regex(regex);
            return rx.Matches(content);
        }
        //下载页面中的图片
        static string DownImage(string html)
        {
            //这里要注意一点：CHM里图片的路径用单斜杠表示
            var urls = GetHtmlImageUrls(html);
            using (var client = new WebClient())
            {
                foreach (var url in urls)
                {
                    var fileName = Path.GetFileName(url);
                    //首先判断该图片是常用图片
                    if (sysImage.Contains(fileName))
                    {
                        //自带图片不更名
                        if (File.Exists("./cnblogs/image//" + fileName))
                        {
                            fileName = "/image/" + fileName;
                            html = html.Replace(url, ".." + fileName);
                            continue;
                        }
                        else
                        {
                            //如果是系统自带图片，则保留命名
                            fileName = "/image/" + fileName;
                        }
                    }
                    else //若图片非系统自带，则随机命名，避免重名覆盖
                    {
                        fileName = "/image/" + (imgIndex++) + Path.GetExtension(url);
                    }
                    try
                    {
                        client.DownloadFile(url, ".//cnblogs" + fileName);
                        html = html.Replace(url, ".." + fileName);
                    }
                    catch
                    {

                    }
                }
                return html;
            }
        }
        #endregion
    }
}
